<%-- $Id: tree.jsp 6388 2014-01-07 10:52:36Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Project"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.BasicItem"
	import="net.sf.basedb.core.Annotation"
	import="net.sf.basedb.core.AnnotationType"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.Subtypable"
	import="net.sf.basedb.core.BioPlateType"
	import="net.sf.basedb.core.BioPlate"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.overview.GenericOverview"
	import="net.sf.basedb.util.overview.OverviewUtil"
	import="net.sf.basedb.util.overview.Node"
	import="net.sf.basedb.util.overview.ValidationOptions"
	import="java.util.List"
	import="org.json.simple.JSONArray"
	import="org.json.simple.JSONObject"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%!
void generateSubTree(DbControl dc, Node startNode, JSONObject jsonParent, boolean showFailures)
{
	
	List<Node> children = startNode.getChildren();
	if (children == null) return;
	
	for (Node child : children)
	{
		int errors = child.getNumErrors();
		int warnings = child.getNumWarnings();
		int childErrors = child.getChildErrors();
		int childWarnings = child.getChildWarnings();

		String folderIcon = child.getNodeType() == Node.Type.FOLDER ? "Folder" : "Item";
		String tooltip = "";
		if (child.getItemType() == Item.ANNOTATIONTYPE)
		{
			if (startNode.getItemType() == Item.PROTOCOL)
			{
				folderIcon = "ProtocolParameter";
			}
			else
			{
				try
				{
					AnnotationType at = (AnnotationType)child.getItem(dc);
					if (at.isProtocolParameter()) folderIcon = "ProtocolParameter";
				}
				catch (Throwable t)
				{}
			}
		}
		else if (child.getItemType() == Item.ANNOTATION)
		{
			folderIcon = "Annotation";
			try
			{
				AnnotationType at = ((Annotation)child.getItem(dc)).getAnnotationType();
				if (at.isProtocolParameter()) folderIcon = "ProtocolParameter";
			}
			catch (Throwable t)
			{}
		}
		
		if (showFailures)
		{
			if (errors > 0)
			{
				folderIcon += "Error";
			}
			else if (warnings > 0)
			{
				folderIcon += "Warning";
			}
			else if (childErrors > 0)
			{
				folderIcon += "ChildError";
			}
			else if (childWarnings > 0)
			{
				folderIcon += "ChildWarning";
			}			
			if (warnings > 0 && errors > 0)
			{
				tooltip += errors + " error(s); " + warnings + " warning(s) on this item";
			}
			else if (errors > 0)
			{
				tooltip += errors + " error(s) on this item";
			}
			else if (warnings > 0)
			{
				tooltip += warnings + " warning(s) on this item";
			}
			if (childErrors > 0 && childWarnings > 0)
			{
				if (tooltip.length() > 0) tooltip += "; ";
				tooltip += childErrors + " error(s); " + childWarnings + " warning(s) on child items";
			}
			else if (childErrors > 0)
			{
				if (tooltip.length() > 0) tooltip += "; ";
				tooltip += childErrors + " error(s) on child items";
			}
			else if (childWarnings > 0)
			{
				if (tooltip.length() > 0) tooltip += "; ";
				tooltip += childWarnings + " warning(s) on child items";
			}
		}
		int numChildren = child.getChildren() == null ? 0 : child.getChildren().size();
		String subtitle = null;
		if (child.getNodeType() == Node.Type.FOLDER)
		{
			subtitle = child.getChildren() == null ? "0" : Integer.toString(child.getChildren().size());
		}
		else if (child.getItem() instanceof Subtypable)
		{
			try
			{
				ItemSubtype subtype = ((Subtypable)child.getItem(dc)).getItemSubtype();
				if (subtype != null) subtitle = subtype.getName();
			}
			catch (Throwable t)
			{}
		}
		else if (child.getItem() instanceof BioPlate)
		{
			try
			{
				BioPlateType plateType = ((BioPlate)child.getItem(dc)).getBioPlateType();
				if (plateType != null) subtitle = plateType.getName();
			}
			catch (Throwable t)
			{}
		}
		
		String text = HTML.encodeTags(child.getTitle()) + (subtitle != null ? " <span class=\"subtitle\">(" + subtitle + ")</span>" : "");
		JSONObject jsonChild = newJoustEntry(jsonParent, folderIcon, text, child.getId());
		jsonChild.put("tooltip", tooltip);
		
		if (!child.isChildrenLoaded())
		{
			jsonChild.put("isLazy", 1);
		}
		generateSubTree(dc, child, jsonChild, showFailures);
	}
}


JSONObject newJoustEntry(JSONObject jsonParent, String icon, String text, String id)
{
	JSONObject jsonJoust = new JSONObject();
	jsonJoust.put("icon", icon);
	jsonJoust.put("text", text);
	jsonJoust.put("id", id);
	if (jsonParent != null)
	{
		JSONArray jsonChildren = (JSONArray)jsonParent.get("children");
		if (jsonChildren == null)
		{
			jsonChildren = new JSONArray();
			jsonParent.put("children", jsonChildren);
		}
		jsonChildren.add(jsonJoust);
	}
	return jsonJoust;
}

%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	GenericOverview overview = OverviewUtil.getCurrentOverview(sc);
	boolean showFailures = Values.getBoolean(request.getParameter("show_failures"), false);
	Node rootNode = overview.getRootNode();
	Nameable rootItem = (Nameable)rootNode.getItem(dc);
	String subtitle = null;
	if (rootItem instanceof Subtypable)
	{
		try
		{
			ItemSubtype subtype = ((Subtypable)rootItem).getItemSubtype();
			if (subtype != null) subtitle = subtype.getName();
		}
		catch (Throwable t)
		{}
	}
	String rootIcon = "Home";
	if (showFailures)
	{
		if (rootNode.getNumErrors() > 0 || rootNode.getChildErrors() > 0)
		{
			rootIcon = "Error";
		}
		else if (rootNode.getNumWarnings() > 0 || rootNode.getChildWarnings() > 0)
		{
			rootIcon = "Warning";
		}
	}
	
	// Build the Joust menu tree
	JSONArray jsonJoust = new JSONArray();
	JSONObject jsonRoot = newJoustEntry(null, rootIcon, HTML.encodeTags(rootItem.getName())+ (subtitle != null ? " <span class=\"subtitle\">(" + subtitle + ")</span>" : ""), rootNode.getId());
	jsonRoot.put("isOpen", 1);
	jsonRoot.put("noOutlineIcon", 1);
	
	generateSubTree(dc, rootNode, jsonRoot, showFailures);
	jsonJoust.add(jsonRoot);
	%>
	<base:page title="" type="iframe">
	<base:head scripts="joust-2.js,dragdrop.js,~tree.js" styles="joust-2.css">
	<style>
	.subtitle 
	{
		font-size: 0.85em;
		color: #666666;
	}
	.selected .subtitle 
	{
		color: #E8E8E8;
	}
	</style>
	
	</base:head>
		<base:body>
		<div id="page-data" class="datacontainer"
			data-show-failures="<%=showFailures ? 1 : 0 %>"
		></div>
		<div id="main" class="joust absolutefull auto-init" data-auto-init="drag-support">
			<div id="joust" class="joust absolutefull" 
				data-joust-tree="<%=HTML.encodeTags(jsonJoust.toJSONString()) %>"
				style="overflow: auto;">
			</div>
		</div>
		</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>
